@@ -5,27 +5,21 @@
 
 # Please remember to run "make -C docs html" after update "desc" attributes.
 
+import argparse
 import copy
 import grp
 import inspect
-try:
-    import argparse
-except ImportError:  # python 2.6
-    from . import argparse_compat as argparse
 import os
 import pwd
 import re
+import shlex
 import ssl
 import sys
 import textwrap
-import shlex
 
-from gunicorn import __version__
-from gunicorn import _compat
+from gunicorn import __version__, util
 from gunicorn.errors import ConfigError
 from gunicorn.reloader import reloader_engines
-from gunicorn import six
-from gunicorn import util
 
 KNOWN_SETTINGS = []
 PLATFORM = sys.platform
@@ -65,7 +59,7 @@ class Config(object):
     def __setattr__(self, name, value):
         if name != "settings" and name in self.settings:
             raise AttributeError("Invalid access!")
-        super(Config, self).__setattr__(name, value)
+        super().__setattr__(name, value)
 
     def set(self, name, value):
         if name not in self.settings:
@@ -122,7 +116,7 @@ class Config(object):
     @property
     def address(self):
         s = self.settings['bind'].get()
-        return [util.parse_address(_compat.bytes_to_str(bind)) for bind in s]
+        return [util.parse_address(util.bytes_to_str(bind)) for bind in s]
 
     @property
     def uid(self):
@@ -183,7 +177,7 @@ class Config(object):
             return env
 
         for e in raw_env:
-            s = _compat.bytes_to_str(e)
+            s = util.bytes_to_str(e)
             try:
                 k, v = s.split('=', 1)
             except ValueError:
@@ -216,7 +210,7 @@ class Config(object):
 
         global_conf = {}
         for e in raw_global_conf:
-            s = _compat.bytes_to_str(e)
+            s = util.bytes_to_str(e)
             try:
                 k, v = re.split(r'(?<!\\)=', s, 1)
             except ValueError:
@@ -230,7 +224,7 @@ class Config(object):
 
 class SettingMeta(type):
     def __new__(cls, name, bases, attrs):
-        super_new = super(SettingMeta, cls).__new__
+        super_new = super().__new__
         parents = [b for b in bases if isinstance(b, SettingMeta)]
         if not parents:
             return super_new(cls, name, bases, attrs)
@@ -305,7 +299,7 @@ class Setting(object):
         return self.value
 
     def set(self, val):
-        if not six.callable(self.validator):
+        if not callable(self.validator):
             raise TypeError('Invalid validator: %s' % self.name)
         self.value = self.validator(val)
 
@@ -314,6 +308,15 @@ class Setting(object):
                 self.order < other.order)
     __cmp__ = __lt__
 
+    def __repr__(self):
+        return "<%s.%s object at %x with value %r>" % (
+            self.__class__.__module__,
+            self.__class__.__name__,
+            id(self),
+            self.value,
+        )
+
+
 Setting = SettingMeta('Setting', (Setting,), {})
 
 
@@ -323,7 +326,7 @@ def validate_bool(val):
 
     if isinstance(val, bool):
         return val
-    if not isinstance(val, six.string_types):
+    if not isinstance(val, str):
         raise TypeError("Invalid type for casting: %s" % val)
     if val.lower().strip() == "true":
         return True
@@ -340,7 +343,7 @@ def validate_dict(val):
 
 
 def validate_pos_int(val):
-    if not isinstance(val, six.integer_types):
+    if not isinstance(val, int):
         val = int(val, 0)
     else:
         # Booleans are ints!
@@ -350,10 +353,32 @@ def validate_pos_int(val):
     return val
 
 
+def validate_ssl_version(val):
+    ssl_versions = {}
+    for protocol in [p for p in dir(ssl) if p.startswith("PROTOCOL_")]:
+        ssl_versions[protocol[9:]] = getattr(ssl, protocol)
+    if val in ssl_versions:
+        # string matching PROTOCOL_...
+        return ssl_versions[val]
+
+    try:
+        intval = validate_pos_int(val)
+        if intval in ssl_versions.values():
+            # positive int matching a protocol int constant
+            return intval
+    except (ValueError, TypeError):
+        # negative integer or not an integer
+        # drop this in favour of the more descriptive ValueError below
+        pass
+
+    raise ValueError("Invalid ssl_version: %s. Valid options: %s"
+                     % (val, ', '.join(ssl_versions)))
+
+
 def validate_string(val):
     if val is None:
         return None
-    if not isinstance(val, six.string_types):
+    if not isinstance(val, str):
         raise TypeError("Not a string: %s" % val)
     return val.strip()
 
@@ -371,7 +396,7 @@ def validate_list_string(val):
         return []
 
     # legacy syntax
-    if isinstance(val, six.string_types):
+    if isinstance(val, str):
         val = [val]
 
     return [validate_string(v) for v in val]
@@ -400,7 +425,7 @@ def validate_class(val):
 
 def validate_callable(arity):
     def _validate_callable(val):
-        if isinstance(val, six.string_types):
+        if isinstance(val, str):
             try:
                 mod_name, obj_name = val.rsplit(".", 1)
             except ValueError:
@@ -414,9 +439,9 @@ def validate_callable(arity):
             except AttributeError:
                 raise TypeError("Can not load '%s' from '%s'"
                     "" % (obj_name, mod_name))
-        if not six.callable(val):
-            raise TypeError("Value is not six.callable: %s" % val)
-        if arity != -1 and arity != _compat.get_arity(val):
+        if not callable(val):
+            raise TypeError("Value is not callable: %s" % val)
+        if arity != -1 and arity != util.get_arity(val):
             raise TypeError("Value must have an arity of: %s" % arity)
         return val
     return _validate_callable
@@ -454,7 +479,7 @@ def validate_group(val):
 def validate_post_request(val):
     val = validate_callable(-1)(val)
 
-    largs = _compat.get_arity(val)
+    largs = util.get_arity(val)
     if largs == 4:
         return val
     elif largs == 3:
@@ -541,8 +566,11 @@ class Bind(Setting):
     desc = """\
         The socket to bind.
 
-        A string of the form: ``HOST``, ``HOST:PORT``, ``unix:PATH``. An IP is
-        a valid ``HOST``.
+        A string of the form: ``HOST``, ``HOST:PORT``, ``unix:PATH``,
+        ``fd://FD``. An IP is a valid ``HOST``.
+
+        .. versionchanged:: 20.0
+           Support for ``fd://FD`` got added.
 
         Multiple addresses can be bound. ex.::
 
@@ -606,30 +634,24 @@ class WorkerClass(Setting):
         The default class (``sync``) should handle most "normal" types of
         workloads. You'll want to read :doc:`design` for information on when
         you might want to choose one of the other worker classes. Required
-        libraries may be installed using setuptools' ``extra_require`` feature.
+        libraries may be installed using setuptools' ``extras_require`` feature.
 
         A string referring to one of the following bundled classes:
 
         * ``sync``
-        * ``eventlet`` - Requires eventlet >= 0.9.7 (or install it via 
+        * ``eventlet`` - Requires eventlet >= 0.24.1 (or install it via
           ``pip install gunicorn[eventlet]``)
-        * ``gevent``   - Requires gevent >= 0.13 (or install it via 
+        * ``gevent``   - Requires gevent >= 1.4 (or install it via
           ``pip install gunicorn[gevent]``)
-        * ``tornado``  - Requires tornado >= 0.2 (or install it via 
+        * ``tornado``  - Requires tornado >= 0.2 (or install it via
           ``pip install gunicorn[tornado]``)
         * ``gthread``  - Python 2 requires the futures package to be installed
           (or install it via ``pip install gunicorn[gthread]``)
-        * ``gaiohttp`` - Deprecated.
 
         Optionally, you can provide your own worker by giving Gunicorn a
         Python path to a subclass of ``gunicorn.workers.base.Worker``.
         This alternative syntax will load the gevent class:
         ``gunicorn.workers.ggevent.GeventWorker``.
-
-        .. deprecated:: 19.8
-           The ``gaiohttp`` worker is deprecated. Please use
-           ``aiohttp.worker.GunicornWebWorker`` instead. See
-           :ref:`asyncio-workers` for more information on how to use it.
         """
 
 class WorkerThreads(Setting):
@@ -652,7 +674,7 @@ class WorkerThreads(Setting):
         If it is not defined, the default is ``1``.
 
         This setting only affects the Gthread worker type.
-        
+
         .. note::
            If you try to use the ``sync`` worker type and set the ``threads``
            setting to more than 1, the ``gthread`` worker type will be used
@@ -686,7 +708,7 @@ class MaxRequests(Setting):
     desc = """\
         The maximum number of requests a worker will process before restarting.
 
-        Any value greater than zero will limit the number of requests a work
+        Any value greater than zero will limit the number of requests a worker
         will process before automatically restarting. This is a simple method
         to help limit the damage of memory leaks.
 
@@ -1247,10 +1269,15 @@ class AccessLogFormat(Setting):
         D            request time in microseconds
         L            request time in decimal seconds
         p            process ID
-        {Header}i    request header
-        {Header}o    response header
-        {Variable}e  environment variable
+        {header}i    request header
+        {header}o    response header
+        {variable}e  environment variable
         ===========  ===========
+
+        Use lowercase for header and environment variable names, and put
+        ``{...}x`` names inside ``%(...)s``. For example::
+
+            %({x-forwarded-for}i)s
         """
 
 
@@ -1460,6 +1487,20 @@ class StatsdHost(Setting):
     .. versionadded:: 19.1
     """
 
+# Datadog Statsd (dogstatsd) tags. https://docs.datadoghq.com/developers/dogstatsd/
+class DogstatsdTags(Setting):
+    name = "dogstatsd_tags"
+    section = "Logging"
+    cli = ["--dogstatsd-tags"]
+    meta = "DOGSTATSD_TAGS"
+    default = ""
+    validator = validate_string
+    desc = """\
+    A comma-delimited list of datadog statsd (dogstatsd) tags to append to statsd metrics.
+
+    .. versionadded:: 20
+    """
+
 class StatsdPrefix(Setting):
     name = "statsd_prefix"
     section = "Logging"
@@ -1540,7 +1581,7 @@ class OnStarting(Setting):
     name = "on_starting"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = six.callable
+    type = callable
 
     def on_starting(server):
         pass
@@ -1556,7 +1597,7 @@ class OnReload(Setting):
     name = "on_reload"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = six.callable
+    type = callable
 
     def on_reload(server):
         pass
@@ -1572,7 +1613,7 @@ class WhenReady(Setting):
     name = "when_ready"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = six.callable
+    type = callable
 
     def when_ready(server):
         pass
@@ -1588,7 +1629,7 @@ class Prefork(Setting):
     name = "pre_fork"
     section = "Server Hooks"
     validator = validate_callable(2)
-    type = six.callable
+    type = callable
 
     def pre_fork(server, worker):
         pass
@@ -1605,7 +1646,7 @@ class Postfork(Setting):
     name = "post_fork"
     section = "Server Hooks"
     validator = validate_callable(2)
-    type = six.callable
+    type = callable
 
     def post_fork(server, worker):
         pass
@@ -1622,7 +1663,7 @@ class PostWorkerInit(Setting):
     name = "post_worker_init"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = six.callable
+    type = callable
 
     def post_worker_init(worker):
         pass
@@ -1639,7 +1680,7 @@ class WorkerInt(Setting):
     name = "worker_int"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = six.callable
+    type = callable
 
     def worker_int(worker):
         pass
@@ -1657,7 +1698,7 @@ class WorkerAbort(Setting):
     name = "worker_abort"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = six.callable
+    type = callable
 
     def worker_abort(worker):
         pass
@@ -1677,7 +1718,7 @@ class PreExec(Setting):
     name = "pre_exec"
     section = "Server Hooks"
     validator = validate_callable(1)
-    type = six.callable
+    type = callable
 
     def pre_exec(server):
         pass
@@ -1693,7 +1734,7 @@ class PreRequest(Setting):
     name = "pre_request"
     section = "Server Hooks"
     validator = validate_callable(2)
-    type = six.callable
+    type = callable
 
     def pre_request(worker, req):
         worker.log.debug("%s %s" % (req.method, req.path))
@@ -1710,7 +1751,7 @@ class PostRequest(Setting):
     name = "post_request"
     section = "Server Hooks"
     validator = validate_post_request
-    type = six.callable
+    type = callable
 
     def post_request(worker, req, environ, resp):
         pass
@@ -1727,7 +1768,7 @@ class ChildExit(Setting):
     name = "child_exit"
     section = "Server Hooks"
     validator = validate_callable(2)
-    type = six.callable
+    type = callable
 
     def child_exit(server, worker):
         pass
@@ -1746,7 +1787,7 @@ class WorkerExit(Setting):
     name = "worker_exit"
     section = "Server Hooks"
     validator = validate_callable(2)
-    type = six.callable
+    type = callable
 
     def worker_exit(server, worker):
         pass
@@ -1763,7 +1804,7 @@ class NumWorkersChanged(Setting):
     name = "nworkers_changed"
     section = "Server Hooks"
     validator = validate_callable(3)
-    type = six.callable
+    type = callable
 
     def nworkers_changed(server, new_value, old_value):
         pass
@@ -1862,14 +1903,46 @@ class SSLVersion(Setting):
     name = "ssl_version"
     section = "SSL"
     cli = ["--ssl-version"]
-    validator = validate_pos_int
-    default = ssl.PROTOCOL_SSLv23
+    validator = validate_ssl_version
+
+    if hasattr(ssl, "PROTOCOL_TLS"):
+        default = ssl.PROTOCOL_TLS
+    else:
+        default = ssl.PROTOCOL_SSLv23
+
     desc = """\
     SSL version to use (see stdlib ssl module's)
 
+    .. versionchanged:: 20.0.1
+       The default value has been changed from ``ssl.PROTOCOL_SSLv23`` to
+       ``ssl.PROTOCOL_TLS`` when Python >= 3.6 .
+
+    """
+    default = ssl.PROTOCOL_SSLv23
+    desc = """\
+    SSL version to use.
+
+    ============= ============
+    --ssl-version Description
+    ============= ============
+    SSLv3         SSLv3 is not-secure and is strongly discouraged.
+    SSLv23        Alias for TLS. Deprecated in Python 3.6, use TLS.
+    TLS           Negotiate highest possible version between client/server.
+                  Can yield SSL. (Python 3.6+)
+    TLSv1         TLS 1.0
+    TLSv1_1       TLS 1.1 (Python 3.4+)
+    TLSv1_2       TLS 1.2 (Python 3.4+)
+    TLS_SERVER    Auto-negotiate the highest protocol version like TLS,
+                  but only support server-side SSLSocket connections.
+                  (Python 3.6+)
+    ============= ============
+
     .. versionchanged:: 19.7
        The default value has been changed from ``ssl.PROTOCOL_TLSv1`` to
        ``ssl.PROTOCOL_SSLv23``.
+    .. versionchanged:: 20.0
+       This setting now accepts string names based on ``ssl.PROTOCOL_``
+       constants.
     """
 
 class CertReqs(Setting):
@@ -1916,16 +1989,28 @@ class DoHandshakeOnConnect(Setting):
     """
 
 
-if sys.version_info >= (2, 7):
-    class Ciphers(Setting):
-        name = "ciphers"
-        section = "SSL"
-        cli = ["--ciphers"]
-        validator = validate_string
-        default = 'TLSv1'
-        desc = """\
-        Ciphers to use (see stdlib ssl module's)
-        """
+class Ciphers(Setting):
+    name = "ciphers"
+    section = "SSL"
+    cli = ["--ciphers"]
+    validator = validate_string
+    default = None
+    desc = """\
+    SSL Cipher suite to use, in the format of an OpenSSL cipher list.
+
+    By default we use the default cipher list from Python's ``ssl`` module,
+    which contains ciphers considered strong at the time of each Python
+    release.
+
+    As a recommended alternative, the Open Web App Security Project (OWASP)
+    offers `a vetted set of strong cipher strings rated A+ to C-
+    <https://www.owasp.org/index.php/TLS_Cipher_String_Cheat_Sheet>`_.
+    OWASP provides details on user-agent compatibility at each security level.
+
+    See the `OpenSSL Cipher List Format Documentation
+    <https://www.openssl.org/docs/manmaster/man1/ciphers.html#CIPHER-LIST-FORMAT>`_
+    for details on the format of an OpenSSL cipher list.
+    """
 
 
 class PasteGlobalConf(Setting):
@@ -1948,3 +2033,20 @@ class PasteGlobalConf(Setting):
 
         .. versionadded:: 19.7
         """
+
+
+class StripHeaderSpaces(Setting):
+    name = "strip_header_spaces"
+    section = "Server Mechanics"
+    cli = ["--strip-header-spaces"]
+    validator = validate_bool
+    action = "store_true"
+    default = False
+    desc = """\
+        Strip spaces present between the header name and the the ``:``.
+        
+        This is known to induce vulnerabilities and is not compliant with the HTTP/1.1 standard.
+        See https://portswigger.net/research/http-desync-attacks-request-smuggling-reborn.
+        
+        Use with care and only if necessary.
+        """
