@@ -2,30 +2,32 @@
 #
 # This file is part of gunicorn released under the MIT license.
 # See the NOTICE for more information.
-
-from __future__ import print_function
-
+import ast
+import ctypes.util
 import email.utils
+import errno
 import fcntl
+import html
+import importlib
+import inspect
 import io
+import logging
 import os
-import pkg_resources
 import pwd
 import random
+import re
 import socket
 import sys
 import textwrap
 import time
 import traceback
-import inspect
-import errno
 import warnings
-import cgi
-import logging
+
+import pkg_resources
 
 from gunicorn.errors import AppImportError
-from gunicorn.six import text_type
 from gunicorn.workers import SUPPORTED_WORKERS
+import urllib.parse
 
 REDIRECT_TO = getattr(os, 'devnull', '/dev/null')
 
@@ -50,44 +52,7 @@ try:
         setproctitle("gunicorn: %s" % title)
 except ImportError:
     def _setproctitle(title):
-        return
-
-
-try:
-    from importlib import import_module
-except ImportError:
-    def _resolve_name(name, package, level):
-        """Return the absolute name of the module to be imported."""
-        if not hasattr(package, 'rindex'):
-            raise ValueError("'package' not set to a string")
-        dot = len(package)
-        for x in range(level, 1, -1):
-            try:
-                dot = package.rindex('.', 0, dot)
-            except ValueError:
-                msg = "attempted relative import beyond top-level package"
-                raise ValueError(msg)
-        return "%s.%s" % (package[:dot], name)
-
-    def import_module(name, package=None):
-        """Import a module.
-
-The 'package' argument is required when performing a relative import. It
-specifies the package to use as the anchor point from which to resolve the
-relative import to an absolute import.
-
-"""
-        if name.startswith('.'):
-            if not package:
-                raise TypeError("relative imports require the 'package' argument")
-            level = 0
-            for character in name:
-                if character != '.':
-                    break
-                level += 1
-            name = _resolve_name(name[level:], package, level)
-        __import__(name)
-        return sys.modules[name]
+        pass
 
 
 def load_class(uri, default="gunicorn.workers.sync.SyncWorker",
@@ -131,7 +96,7 @@ def load_class(uri, default="gunicorn.workers.sync.SyncWorker",
         klass = components.pop(-1)
 
         try:
-            mod = import_module('.'.join(components))
+            mod = importlib.import_module('.'.join(components))
         except:
             exc = traceback.format_exc()
             msg = "class uri %r invalid or not found: \n\n[%s]"
@@ -139,6 +104,23 @@ def load_class(uri, default="gunicorn.workers.sync.SyncWorker",
         return getattr(mod, klass)
 
 
+positionals = (
+    inspect.Parameter.POSITIONAL_ONLY,
+    inspect.Parameter.POSITIONAL_OR_KEYWORD,
+)
+
+
+def get_arity(f):
+    sig = inspect.signature(f)
+    arity = 0
+
+    for param in sig.parameters.values():
+        if param.kind in positionals:
+            arity += 1
+
+    return arity
+
+
 def get_username(uid):
     """ get the username for a user id"""
     return pwd.getpwuid(uid).pw_name
@@ -160,7 +142,7 @@ def set_owner_process(uid, gid, initgroups=False):
 
         if initgroups:
             os.initgroups(username, gid)
-        else:
+        elif gid != os.getgid():
             os.setgid(gid)
 
     if uid:
@@ -168,13 +150,12 @@ def set_owner_process(uid, gid, initgroups=False):
 
 
 def chown(path, uid, gid):
-    gid = abs(gid) & 0x7FFFFFFF  # see note above.
     os.chown(path, uid, gid)
 
 
 if sys.platform.startswith("win"):
     def _waitfor(func, pathname, waitall=False):
-        # Peform the operation
+        # Perform the operation
         func(pathname)
         # Now setup the wait loop
         if waitall:
@@ -190,7 +171,7 @@ if sys.platform.startswith("win"):
         # required when contention occurs.
         timeout = 0.001
         while timeout < 1.0:
-            # Note we are only testing for the existance of the file(s) in
+            # Note we are only testing for the existence of the file(s) in
             # the contents of the directory regardless of any security or
             # access rights.  If we have made it this far, we have sufficient
             # permissions to do that much using Python's equivalent of the
@@ -198,7 +179,7 @@ if sys.platform.startswith("win"):
             # Other Windows APIs can fail or give incorrect results when
             # dealing with files that are pending deletion.
             L = os.listdir(dirname)
-            if not (L if waitall else name in L):
+            if not L if waitall else name in L:
                 return
             # Increase the timeout and try again
             time.sleep(timeout)
@@ -231,36 +212,35 @@ def is_ipv6(addr):
     return True
 
 
-def parse_address(netloc, default_port=8000):
-    if netloc.startswith("unix://"):
-        return netloc.split("unix://")[1]
+def parse_address(netloc, default_port='8000'):
+    if re.match(r'unix:(//)?', netloc):
+        return re.split(r'unix:(//)?', netloc)[-1]
 
-    if netloc.startswith("unix:"):
-        return netloc.split("unix:")[1]
+    if netloc.startswith("fd://"):
+        fd = netloc[5:]
+        try:
+            return int(fd)
+        except ValueError:
+            raise RuntimeError("%r is not a valid file descriptor." % fd) from None
 
     if netloc.startswith("tcp://"):
         netloc = netloc.split("tcp://")[1]
+    host, port = netloc, default_port
 
-    # get host
     if '[' in netloc and ']' in netloc:
-        host = netloc.split(']')[0][1:].lower()
+        host = netloc.split(']')[0][1:]
+        port = (netloc.split(']:') + [default_port])[1]
     elif ':' in netloc:
-        host = netloc.split(':')[0].lower()
+        host, port = (netloc.split(':') + [default_port])[:2]
     elif netloc == "":
-        host = "0.0.0.0"
-    else:
-        host = netloc.lower()
+        host, port = "0.0.0.0", default_port
 
-    #get port
-    netloc = netloc.split(']')[-1]
-    if ":" in netloc:
-        port = netloc.split(':', 1)[1]
-        if not port.isdigit():
-            raise RuntimeError("%r is not a valid port number." % port)
+    try:
         port = int(port)
-    else:
-        port = default_port
-    return (host, port)
+    except ValueError:
+        raise RuntimeError("%r is not a valid port number." % port)
+
+    return host.lower(), port
 
 
 def close_on_exec(fd):
@@ -293,7 +273,7 @@ except ImportError:
 
 
 def write_chunk(sock, data):
-    if isinstance(data, text_type):
+    if isinstance(data, str):
         data = data.encode('utf-8')
     chunk_size = "%X\r\n" % len(data)
     chunk = b"".join([chunk_size.encode('utf-8'), data, b"\r\n"])
@@ -319,7 +299,7 @@ def write_nonblock(sock, data, chunked=False):
 
 
 def write_error(sock, status_int, reason, mesg):
-    html = textwrap.dedent("""\
+    html_error = textwrap.dedent("""\
     <html>
       <head>
         <title>%(reason)s</title>
@@ -329,7 +309,7 @@ def write_error(sock, status_int, reason, mesg):
         %(mesg)s
       </body>
     </html>
-    """) % {"reason": reason, "mesg": cgi.escape(mesg)}
+    """) % {"reason": reason, "mesg": html.escape(mesg)}
 
     http = textwrap.dedent("""\
     HTTP/1.1 %s %s\r
@@ -337,35 +317,110 @@ def write_error(sock, status_int, reason, mesg):
     Content-Type: text/html\r
     Content-Length: %d\r
     \r
-    %s""") % (str(status_int), reason, len(html), html)
+    %s""") % (str(status_int), reason, len(html_error), html_error)
     write_nonblock(sock, http.encode('latin1'))
 
 
+def _called_with_wrong_args(f):
+    """Check whether calling a function raised a ``TypeError`` because
+    the call failed or because something in the function raised the
+    error.
+
+    :param f: The function that was called.
+    :return: ``True`` if the call failed.
+    """
+    tb = sys.exc_info()[2]
+
+    try:
+        while tb is not None:
+            if tb.tb_frame.f_code is f.__code__:
+                # In the function, it was called successfully.
+                return False
+
+            tb = tb.tb_next
+
+        # Didn't reach the function.
+        return True
+    finally:
+        # Delete tb to break a circular reference in Python 2.
+        # https://docs.python.org/2/library/sys.html#sys.exc_info
+        del tb
+
+
 def import_app(module):
     parts = module.split(":", 1)
     if len(parts) == 1:
-        module, obj = module, "application"
+        obj = "application"
     else:
         module, obj = parts[0], parts[1]
 
     try:
-        __import__(module)
+        mod = importlib.import_module(module)
     except ImportError:
         if module.endswith(".py") and os.path.exists(module):
             msg = "Failed to find application, did you mean '%s:%s'?"
             raise ImportError(msg % (module.rsplit(".", 1)[0], obj))
-        else:
-            raise
+        raise
 
-    mod = sys.modules[module]
+    # Parse obj as a single expression to determine if it's a valid
+    # attribute name or function call.
+    try:
+        expression = ast.parse(obj, mode="eval").body
+    except SyntaxError:
+        raise AppImportError(
+            "Failed to parse %r as an attribute name or function call." % obj
+        )
+
+    if isinstance(expression, ast.Name):
+        name = expression.id
+        args = kwargs = None
+    elif isinstance(expression, ast.Call):
+        # Ensure the function name is an attribute name only.
+        if not isinstance(expression.func, ast.Name):
+            raise AppImportError("Function reference must be a simple name: %r" % obj)
+
+        name = expression.func.id
+
+        # Parse the positional and keyword arguments as literals.
+        try:
+            args = [ast.literal_eval(arg) for arg in expression.args]
+            kwargs = {kw.arg: ast.literal_eval(kw.value) for kw in expression.keywords}
+        except ValueError:
+            # literal_eval gives cryptic error messages, show a generic
+            # message with the full expression instead.
+            raise AppImportError(
+                "Failed to parse arguments as literal values: %r" % obj
+            )
+    else:
+        raise AppImportError(
+            "Failed to parse %r as an attribute name or function call." % obj
+        )
 
     is_debug = logging.root.level == logging.DEBUG
     try:
-        app = eval(obj, vars(mod))
-    except NameError:
+        app = getattr(mod, name)
+    except AttributeError:
         if is_debug:
             traceback.print_exception(*sys.exc_info())
-        raise AppImportError("Failed to find application: %r" % module)
+        raise AppImportError("Failed to find attribute %r in %r." % (name, module))
+
+    # If the expression was a function call, call the retrieved object
+    # to get the real application.
+    if args is not None:
+        try:
+            app = app(*args, **kwargs)
+        except TypeError as e:
+            # If the TypeError was due to bad arguments to the factory
+            # function, show Python's nice error message without a
+            # traceback.
+            if _called_with_wrong_args(app):
+                raise AppImportError(
+                    "".join(traceback.format_exception_only(TypeError, e)).strip()
+                )
+
+            # Otherwise it was raised from within the function, show the
+            # full traceback.
+            raise
 
     if app is None:
         raise AppImportError("Failed to find application object: %r" % obj)
@@ -417,7 +472,7 @@ def daemonize(enable_stdio_inheritance=False):
         os.umask(0o22)
 
         # In both the following any file descriptors above stdin
-        # stdout and stderr are left untouched. The inheritence
+        # stdout and stderr are left untouched. The inheritance
         # option simply allows one to have output go to a file
         # specified by way of shell redirection when not wanting
         # to use --error-log option.
@@ -503,11 +558,12 @@ def to_bytestring(value, encoding="utf8"):
     """Converts a string argument to a byte string"""
     if isinstance(value, bytes):
         return value
-    if not isinstance(value, text_type):
+    if not isinstance(value, str):
         raise TypeError('%r is not a string' % value)
 
     return value.encode(encoding)
 
+
 def has_fileno(obj):
     if not hasattr(obj, "fileno"):
         return False
@@ -545,3 +601,69 @@ def make_fail_app(msg):
         return [msg]
 
     return app
+
+
+def split_request_uri(uri):
+    if uri.startswith("//"):
+        # When the path starts with //, urlsplit considers it as a
+        # relative uri while the RFC says we should consider it as abs_path
+        # http://www.w3.org/Protocols/rfc2616/rfc2616-sec5.html#sec5.1.2
+        # We use temporary dot prefix to workaround this behaviour
+        parts = urllib.parse.urlsplit("." + uri)
+        return parts._replace(path=parts.path[1:])
+
+    return urllib.parse.urlsplit(uri)
+
+
+# From six.reraise
+def reraise(tp, value, tb=None):
+    try:
+        if value is None:
+            value = tp()
+        if value.__traceback__ is not tb:
+            raise value.with_traceback(tb)
+        raise value
+    finally:
+        value = None
+        tb = None
+
+
+def bytes_to_str(b):
+    if isinstance(b, str):
+        return b
+    return str(b, 'latin1')
+
+
+def unquote_to_wsgi_str(string):
+    return urllib.parse.unquote_to_bytes(string).decode('latin-1')
+
+
+def _findWalk_ldpath(name):
+    def _is_elf(filepath):
+        try:
+            with open(filepath, 'rb') as fh:
+                return fh.read(4) == b'\x7fELF'
+        except:
+            return False
+    from glob import glob
+    if os.path.isabs(name):
+        return name
+
+    # search LD_LIBRARY_PATH list
+    paths = ['/lib', '/usr/local/lib', '/usr/lib']
+    if 'LD_LIBRARY_PATH' in os.environ:
+        paths = os.environ['LD_LIBRARY_PATH'].split(':') + paths
+
+    for d in paths:
+        f = os.path.join(d, name)
+        if _is_elf(f):
+            return os.path.basename(f)
+        prefix = os.path.join(d, 'lib'+name)
+        for suffix in ['so', 'so.*', '*.so.*', 'a']:
+            for f in glob('{0}.{1}'.format(prefix, suffix)):
+                if _is_elf(f) or suffix == 'a':
+                    return os.path.basename(f)
+
+
+def find_library(name):
+    return ctypes.util.find_library(name) or _findWalk_ldpath(name)
